@@ -18,10 +18,12 @@
 
 #include "audiooutput.h"
 #include "backend.h"
+#include "debug.h"
 #include "devicemanager.h"
 #include "mediaobject.h"
 #include "gsthelper.h"
 #include <phonon/audiooutput.h>
+#include <phonon/pulsesupport.h>
 
 #include <QtCore/QStringBuilder>
 
@@ -29,8 +31,6 @@
 #include <gst/gstghostpad.h>
 #include <gst/gstutils.h>
 
-QT_BEGIN_NAMESPACE
-
 namespace Phonon
 {
 namespace Gstreamer
@@ -116,70 +116,36 @@ void AudioOutput::setVolume(qreal newVolume)
     emit volumeChanged(newVolume);
 }
 
+/*
+ * Reimp
+ */
 bool AudioOutput::setOutputDevice(int newDevice)
 {
-    m_backend->logMessage(Q_FUNC_INFO % QString::number(newDevice), Backend::Info, this);
-
-    if (newDevice == m_device)
-        return true;
-
-    if (root()) {
-        root()->saveState();
-        if (root()->pipeline()->setState(GST_STATE_READY) == GST_STATE_CHANGE_FAILURE)
-            return false;
+    const AudioOutputDevice device = AudioOutputDevice::fromIndex(newDevice);
+    if (!device.isValid()) {
+        error() << Q_FUNC_INFO << "Unable to find the output device with index" << newDevice;
+        return false;
     }
-
-    bool success = false;
-    if (m_audioSink &&  newDevice >= 0) {
-        // Save previous state
-        GstState oldState = GST_STATE(m_audioSink);
-        const QByteArray oldDeviceValue = GstHelper::property(m_audioSink, "device");
-        const QByteArray deviceId = m_backend->deviceManager()->gstId(newDevice);
-        m_device = newDevice;
-
-        // We test if the device can be opened by checking if it can go from NULL to READY state
-        gst_element_set_state(m_audioSink, GST_STATE_NULL);
-        success = GstHelper::setProperty(m_audioSink, "device", deviceId);
-        if (success) {
-            success = (gst_element_set_state(m_audioSink, oldState) == GST_STATE_CHANGE_SUCCESS);
-        }
-        if (!success) { // Revert state
-            m_backend->logMessage(Q_FUNC_INFO %
-                                  QLatin1Literal(" Failed to change device ") %
-                                  deviceId, Backend::Info, this);
-
-            GstHelper::setProperty(m_audioSink, "device", oldDeviceValue);
-            gst_element_set_state(m_audioSink, oldState);
-        } else {
-            m_backend->logMessage(Q_FUNC_INFO %
-                                  QLatin1String(" Successfully changed device ") %
-                                  deviceId, Backend::Info, this);
-        }
-
-        // Note the stopped state should not really be necessary, but seems to be required to
-        // properly reset after changing the audio state
-        if (root()) {
-            QMetaObject::invokeMethod(root(), "setState", Qt::QueuedConnection, Q_ARG(State, StoppedState));
-            root()->resumeState();
-        }
-    }
-    return success;
+    return setOutputDevice(device);
 }
 
 #if (PHONON_VERSION >= PHONON_VERSION_CHECK(4, 2, 0))
 bool AudioOutput::setOutputDevice(const AudioOutputDevice &newDevice)
 {
-    m_backend->logMessage(Q_FUNC_INFO, Backend::Info, this);
+    debug() << Q_FUNC_INFO;
     if (!m_audioSink || !newDevice.isValid()) {
         return false;
     }
-    const QVariant driver = newDevice.property("driver");
-    if (!driver.isValid()) {
-        return setOutputDevice(newDevice.index());
-    }
-    if (newDevice.index() == m_device) {
+
+    const QVariant dalProperty = newDevice.property("deviceAccessList");
+    if (!dalProperty.isValid())
+        return false;
+    const DeviceAccessList deviceAccessList = dalProperty.value<DeviceAccessList>();
+    if (deviceAccessList.isEmpty())
+        return false;
+
+    if (newDevice.index() == m_device)
         return true;
-    }
 
     if (root()) {
         root()->saveState();
@@ -191,47 +157,13 @@ bool AudioOutput::setOutputDevice(const AudioOutputDevice &newDevice)
     const GstState oldState = GST_STATE(m_audioSink);
     const QByteArray oldDeviceValue = GstHelper::property(m_audioSink, "device");
 
-    const QByteArray sinkName = GstHelper::property(m_audioSink, "name");
-    if (sinkName == QByteArray("alsasink")) {
-        if (driver.toByteArray() != QByteArray("alsa")) {
-            return false;
+    foreach (const DeviceAccess &deviceAccess, deviceAccessList) {
+        if (setOutputDevice(deviceAccess.first, deviceAccess.second, oldState)) {
+            m_device = newDevice.index();
+            return true;
         }
     }
 
-    const QVariant deviceIdsProperty = newDevice.property("deviceIds");
-    QStringList deviceIds;
-    if (deviceIdsProperty.type() == QVariant::StringList) {
-        deviceIds = deviceIdsProperty.toStringList();
-    } else if (deviceIdsProperty.type() == QVariant::String) {
-        deviceIds += deviceIdsProperty.toString();
-    }
-
-    // We test if the device can be opened by checking if it can go from NULL to READY state
-    foreach (const QString &deviceId, deviceIds) {
-        gst_element_set_state(m_audioSink, GST_STATE_NULL);
-        if (GstHelper::setProperty(m_audioSink, "device", deviceId.toUtf8())) {
-            m_backend->logMessage(Q_FUNC_INFO % QLatin1Literal("setProperty(device,") %
-                                  deviceId % QLatin1Literal(") succeeded"), Backend::Info, this);
-            if (gst_element_set_state(m_audioSink, oldState) == GST_STATE_CHANGE_SUCCESS) {
-                m_backend->logMessage(Q_FUNC_INFO % QLatin1Literal("go to old state on device") %
-                                      deviceId % QLatin1Literal(" succeeded"), Backend::Info, this);
-                m_device = newDevice.index();
-                if (root()) {
-                    QMetaObject::invokeMethod(root(), "setState",
-                                              Qt::QueuedConnection,
-                                              Q_ARG(State, StoppedState));
-                    root()->resumeState();
-                }
-                return true;
-            } else {
-                m_backend->logMessage(Q_FUNC_INFO % QLatin1Literal("go to old state on device") %
-                                      deviceId % QLatin1Literal(" failed"), Backend::Info, this);
-            }
-        } else {
-            m_backend->logMessage(Q_FUNC_INFO % QLatin1Literal("setProperty(device,") %
-                                  deviceId % QLatin1Literal(") failed"), Backend::Info, this);
-        }
-    }
     // Revert state
     GstHelper::setProperty(m_audioSink, "device", oldDeviceValue);
     gst_element_set_state(m_audioSink, oldState);
@@ -244,10 +176,64 @@ bool AudioOutput::setOutputDevice(const AudioOutputDevice &newDevice)
 
     return false;
 }
+
+bool AudioOutput::setOutputDevice(const QByteArray &driver, const QString &deviceId, const GstState oldState)
+{
+    const QByteArray sinkName = GstHelper::property(m_audioSink, "name");
+    if (sinkName == QByteArray("alsasink")) {
+        if (driver != QByteArray("alsa")) {
+            return false;
+        }
+    }
+
+    // We test if the device can be opened by checking if it can go from NULL to READY state
+    gst_element_set_state(m_audioSink, GST_STATE_NULL);
+    if (GstHelper::setProperty(m_audioSink, "device", deviceId.toUtf8())) {
+        debug() << Q_FUNC_INFO << "setProperty( device," << deviceId << ") succeeded";
+        if (gst_element_set_state(m_audioSink, oldState) == GST_STATE_CHANGE_SUCCESS) {
+            debug() << Q_FUNC_INFO << "go to old state on device" << deviceId << "succeeded";
+            if (root()) {
+                QMetaObject::invokeMethod(root(), "setState",
+                                            Qt::QueuedConnection,
+                                            Q_ARG(State, StoppedState));
+                root()->resumeState();
+            }
+            return true;
+        } else {
+            error() << Q_FUNC_INFO << "go to old state on device" << deviceId << "failed";
+        }
+    } else {
+        error() << Q_FUNC_INFO << "setProperty( device," << deviceId << ") failed";
+    }
+
+    return false;
+}
+#endif
+
+#if (PHONON_VERSION >= PHONON_VERSION_CHECK(4, 6, 50))
+void AudioOutput::setStreamUuid(QString uuid)
+{
+    m_streamUuid = uuid;
+#warning this really needs a check for pulsesink as well
+    if (g_object_class_find_property(G_OBJECT_GET_CLASS(m_audioSink), "stream-properties")) {
+        const QHash<QString, QString> streamProperties = PulseSupport::getInstance()->streamProperties(uuid);
+        GstStructure *properties = gst_structure_empty_new("props");
+
+        QHashIterator<QString, QString> it(streamProperties);
+        while (it.hasNext()) {
+            it.next();
+            gst_structure_set(properties,
+                              it.key().toUtf8().constData(), G_TYPE_STRING, it.value().toUtf8().constData(),
+                              NULL);
+        }
+
+        Q_ASSERT(properties);
+        g_object_set (m_audioSink, "stream-properties", properties, NULL);
+        gst_structure_free(properties);
+    }
+}
 #endif
 
 }
 } //namespace Phonon::Gstreamer
-
-QT_END_NAMESPACE
 #include "moc_audiooutput.cpp"
