@@ -15,10 +15,12 @@
     along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <gst/interfaces/propertyprobe.h>
-#include "phonon-config-gstreamer.h"
 #include "devicemanager.h"
+
+#include <gst/interfaces/propertyprobe.h>
+#include "phonon-config-gstreamer.h" // krazy:exclude=includes
 #include "backend.h"
+#include "debug.h"
 #include "gsthelper.h"
 #include "videowidget.h"
 #ifdef OPENGL_FOUND
@@ -35,74 +37,130 @@
  * active output devices
  */
 
-QT_BEGIN_NAMESPACE
-
 namespace Phonon
 {
 namespace Gstreamer
 {
 
-VideoCaptureDevice::VideoCaptureDevice(DeviceManager *manager, const QByteArray &gstId)
-        : gstId(gstId)
+/*
+ * Device Info
+ */
+
+DeviceInfo::DeviceInfo(DeviceManager *manager, const QByteArray &deviceId,
+                       quint16 caps, bool isAdvanced)
+        : m_isAdvanced(isAdvanced), m_capabilities(caps)
 {
-    id = manager->allocateDeviceId();
-    icon = "camera-web";
+    // Get an unique integer id for each device
+    static int deviceCounter = 0;
+    m_id = deviceCounter ++;
 
-    //get name from device
-    if (gstId == "default") {
-        description = "Default video capture device";
-    } else {
-        GstElement *dev = gst_element_factory_make("v4l2src", NULL);
-
-        if (dev) {
-            gchar *deviceDescription = NULL;
-
-            if (GST_IS_PROPERTY_PROBE(dev) && gst_property_probe_get_property( GST_PROPERTY_PROBE(dev), "device" ) ) {
-                g_object_set (G_OBJECT(dev), "device", gstId.constData(), NULL);
-                g_object_get (G_OBJECT(dev), "device-name", &deviceDescription, NULL);
-                description = QByteArray(deviceDescription);
-                g_free (deviceDescription);
-                gst_element_set_state(dev, GST_STATE_NULL);
-                gst_object_unref (dev);
-            }
+    if (caps & VideoCapture) {
+        // Get a preferred name from the device
+        if (deviceId == "default") {
+            m_name = "Default";
+            m_description = "Default video capture device";
+        } else {
+            GstElement *dev = gst_element_factory_make("v4l2src", NULL);
+            useGstElement(dev, deviceId);
         }
     }
+
+    if (caps & AudioOutput) {
+        // This should never be called when PulseAudio is active.
+        Q_ASSERT(!PulseSupport::getInstance()->isActive());
+
+        // Get a preferred name from the device
+        if (deviceId == "default") {
+            m_name = "Default";
+            m_description = "Default audio device";
+        } else {
+            GstElement *aSink = manager->createAudioSink();
+            useGstElement(aSink, deviceId);
+        }
+    }
+
+    // A default device should never be advanced
+    if (deviceId == "default")
+        m_isAdvanced = false;
+}
+
+void DeviceInfo::useGstElement(GstElement *element, const QByteArray &deviceId)
+{
+    if (!element)
+        return;
+
+    gchar *deviceName = NULL;
+    if (GST_IS_PROPERTY_PROBE(element) && gst_property_probe_get_property(GST_PROPERTY_PROBE(element), "device")) {
+        g_object_set(G_OBJECT(element), "device", deviceId.constData(), NULL);
+        g_object_get(G_OBJECT(element), "device-name", &deviceName, NULL);
+        m_name = QString(deviceName);
+
+        if (m_description.isEmpty()) {
+            // Construct a description by using the factory name and the device id
+            GstElementFactory *factory = gst_element_get_factory(element);
+            const gchar *factoryName = gst_element_factory_get_longname(factory);
+            m_description = QString(factoryName) + ": " + deviceId;
+        }
+
+        g_free(deviceName);
+        gst_element_set_state(element, GST_STATE_NULL);
+        gst_object_unref(element);
+    }
 }
 
-AudioDevice::AudioDevice(DeviceManager *manager, const QByteArray &gstId)
-        : gstId(gstId)
+int DeviceInfo::id() const
 {
-    // This should never be called when PulseAudio is active.
-    Q_ASSERT(!PulseSupport::getInstance()->isActive());
-
-    id = manager->allocateDeviceId();
-    icon = "audio-card";
-
-    //get name from device
-    if (gstId == "default") {
-        description = "Default audio device";
-    } else {
-        GstElement *aSink= manager->createAudioSink();
-
-        if (aSink) {
-            gchar *deviceDescription = NULL;
-
-            if (GST_IS_PROPERTY_PROBE(aSink) && gst_property_probe_get_property( GST_PROPERTY_PROBE(aSink), "device" ) ) {
-                g_object_set (G_OBJECT(aSink), "device", gstId.constData(), NULL);
-                g_object_get (G_OBJECT(aSink), "device-name", &deviceDescription, NULL);
-                description = QByteArray(deviceDescription);
-                g_free (deviceDescription);
-                gst_element_set_state(aSink, GST_STATE_NULL);
-                gst_object_unref (aSink);
-            }
-        }
-    }
+    return m_id;
 }
 
+const QString& DeviceInfo::name() const
+{
+    return m_name;
+}
+
+const QString& DeviceInfo::description() const
+{
+    return m_description;
+}
+
+bool DeviceInfo::isAdvanced() const
+{
+    return m_isAdvanced;
+}
+
+void DeviceInfo::setAdvanced(bool advanced)
+{
+    m_isAdvanced = advanced;
+}
+
+const DeviceAccessList& DeviceInfo::accessList() const
+{
+    return m_accessList;
+}
+
+void DeviceInfo::addAccess(const DeviceAccess& access)
+{
+    m_accessList.append(access);
+}
+
+quint16 DeviceInfo::capabilities() const
+{
+    return m_capabilities;
+}
+
+void DeviceInfo::setCapabilities(quint16 cap)
+{
+    m_capabilities = cap;
+}
+
+
+/*
+ * Device Manager
+ */
+
 DeviceManager::DeviceManager(Backend *backend)
         : QObject(backend)
         , m_backend(backend)
-        , m_audioDeviceCounter(0)
 {
     QSettings settings(QLatin1String("Trolltech"));
     settings.beginGroup(QLatin1String("Qt"));
@@ -135,8 +193,6 @@ DeviceManager::DeviceManager(Backend *backend)
 
 DeviceManager::~DeviceManager()
 {
-    m_audioDeviceList.clear();
-    m_videoCaptureDeviceList.clear();
 }
 
 /***
@@ -211,7 +267,7 @@ GstElement *DeviceManager::createAudioSink(Category category)
         if (!qgetenv("GNOME_DESKTOP_SESSION_ID").isEmpty()) {
             sink = createGNOMEAudioSink(category);
             if (canOpenDevice(sink))
-                    m_backend->logMessage("AudioOutput using gconf audio sink");
+                debug() << "AudioOutput using gconf audio sink";
             else if (sink) {
                 gst_object_unref(sink);
                 sink = 0;
@@ -221,7 +277,7 @@ GstElement *DeviceManager::createAudioSink(Category category)
         if (!sink) {
             sink = gst_element_factory_make ("alsasink", NULL);
             if (canOpenDevice(sink))
-                    m_backend->logMessage("AudioOutput using alsa audio sink");
+                debug() << "AudioOutput using alsa audio sink";
             else if (sink) {
                 gst_object_unref(sink);
                 sink = 0;
@@ -231,7 +287,7 @@ GstElement *DeviceManager::createAudioSink(Category category)
         if (!sink) {
             sink = gst_element_factory_make ("autoaudiosink", NULL);
             if (canOpenDevice(sink))
-                    m_backend->logMessage("AudioOutput using auto audio sink");
+                debug() << "AudioOutput using auto audio sink";
             else if (sink) {
                 gst_object_unref(sink);
                 sink = 0;
@@ -241,7 +297,7 @@ GstElement *DeviceManager::createAudioSink(Category category)
         if (!sink) {
             sink = gst_element_factory_make ("osssink", NULL);
             if (canOpenDevice(sink))
-                    m_backend->logMessage("AudioOutput using oss audio sink");
+                debug() << "AudioOutput using oss audio sink";
             else if (sink) {
                 gst_object_unref(sink);
                 sink = 0;
@@ -252,7 +308,7 @@ GstElement *DeviceManager::createAudioSink(Category category)
     } else if (!m_audioSink.isEmpty()) { //Use a custom sink
         sink = gst_element_factory_make (m_audioSink, NULL);
         if (canOpenDevice(sink))
-                m_backend->logMessage(QString("AudioOutput using %0").arg(QString::fromUtf8(m_audioSink)));
+            debug() << "AudioOutput using" << QString::fromUtf8(m_audioSink);
         else {
             if (sink) {
                 gst_object_unref(sink);
@@ -261,7 +317,7 @@ GstElement *DeviceManager::createAudioSink(Category category)
             if ("pulsesink" == m_audioSink) {
                 // We've tried to use PulseAudio support, but the GST plugin
                 // doesn't exits. Let's try again, but not use PA support this time.
-                    m_backend->logMessage("PulseAudio support failed. Falling back to 'auto'");
+                warning() << "PulseAudio support failed. Falling back to 'auto'";
                 PulseSupport::getInstance()->enable(false);
                 m_audioSink = "auto";
                 sink = createAudioSink();
@@ -272,7 +328,7 @@ GstElement *DeviceManager::createAudioSink(Category category)
     if (!sink) { //no suitable sink found so we'll make a fake one
         sink = gst_element_factory_make("fakesink", NULL);
         if (sink) {
-            m_backend->logMessage("AudioOutput Using fake audio sink");
+            warning() << "AudioOutput Using fake audio sink";
             //without sync the sink will pull the pipeline as fast as the CPU allows
             g_object_set (G_OBJECT (sink), "sync", TRUE, NULL);
         }
@@ -306,68 +362,74 @@ AbstractRenderer *DeviceManager::createVideoRenderer(VideoWidget *parent)
 }
 #endif //QT_NO_PHONON_VIDEO
 
-/**
- * Allocate a device id for a new audio device
- */
-int DeviceManager::allocateDeviceId()
+QList<int> DeviceManager::deviceIds(ObjectDescriptionType type)
 {
-    return m_audioDeviceCounter++;
+    DeviceInfo::Capability capability = DeviceInfo::None;
+    switch (type) {
+    case Phonon::AudioOutputDeviceType:
+        capability = DeviceInfo::AudioOutput;
+        break;
+    case Phonon::AudioCaptureDeviceType:
+        capability = DeviceInfo::AudioCapture;
+        break;
+    case Phonon::VideoCaptureDeviceType:
+        capability = DeviceInfo::VideoCapture;
+        break;
+    default: ;
+    }
+
+    QList<int> ids;
+    foreach (const DeviceInfo &device, m_devices) {
+        if (device.capabilities() & capability)
+            ids.append(device.id());
+    }
+
+    return ids;
 }
 
-
-/**
- * Returns a positive device id or -1 if device does not exist
- *
- * The gstId is typically in the format hw:1,0
- */
-int DeviceManager::deviceId(const QByteArray &gstId) const
+QHash<QByteArray, QVariant> DeviceManager::deviceProperties(int id)
 {
-    for (int i = 0 ; i < m_audioDeviceList.size() ; ++i) {
-        if (m_audioDeviceList[i].gstId == gstId) {
-            return m_audioDeviceList[i].id;
+    QHash<QByteArray, QVariant> properties;
+
+    foreach (const DeviceInfo &device, m_devices) {
+        if (device.id() == id) {
+            properties.insert("name", device.name());
+            properties.insert("description", device.description());
+            properties.insert("isAdvanced", device.isAdvanced());
+            properties.insert("deviceAccessList", QVariant::fromValue<Phonon::DeviceAccessList>(device.accessList()));
+            properties.insert("discovererIcon", QLatin1String("phonon-gstreamer"));
+
+            if (device.capabilities() & DeviceInfo::AudioOutput) {
+                properties.insert("icon", QLatin1String("audio-card"));
+            }
+
+            if (device.capabilities() & DeviceInfo::AudioCapture) {
+                properties.insert("hasaudio", true);
+                properties.insert("icon", QLatin1String("audio-input-microphone"));
+            }
+
+            if (device.capabilities() & DeviceInfo::VideoCapture) {
+                properties.insert("hasvideo", true);
+                properties.insert("icon", QLatin1String("camera-web"));
+            }
+            break;
         }
     }
-    for (int i = 0 ; i < m_videoCaptureDeviceList.size() ; ++i) {
-        if (m_videoCaptureDeviceList[i].gstId == gstId) {
-            return m_videoCaptureDeviceList[i].id;
-        }
-    }
-    return -1;
+
+    return properties;
 }
 
 /**
- * Returns a gstId or "default" if device does not exist
- *
- * The gstId is typically in the format hw:1,0
+ * \param id The identifier for the device
+ * \return Pointer to DeviceInfo, or NULL if the id is invalid
  */
-const QByteArray DeviceManager::gstId(int deviceId)
+const DeviceInfo *DeviceManager::device(int id) const
 {
-    if (!PulseSupport::getInstance()->isActive()) {
-        AudioDevice *ad = audioDevice(deviceId);
-        if (ad)
-            return QByteArray(ad->gstId);
+    for (int i = 0; i < m_devices.size(); i ++) {
+        if (m_devices[i].id() == id)
+            return &m_devices[i];
     }
-    return QByteArray("default");
-}
 
-/**
-* Get the AudioDevice for a given device id
-*/
-AudioDevice* DeviceManager::audioDevice(int id)
-{
-    for (int i = 0 ; i < m_audioDeviceList.size() ; ++i) {
-        if (m_audioDeviceList[i].id == id)
-            return &m_audioDeviceList[i];
-    }
-    return NULL;
-}
-
-VideoCaptureDevice* DeviceManager::videoCaptureDevice(int id)
-{
-    for (int i = 0 ; i < m_videoCaptureDeviceList.size() ; ++i) {
-        if (m_videoCaptureDeviceList[i].id == id)
-            return &m_videoCaptureDeviceList[i];
-    }
     return NULL;
 }
 
@@ -376,101 +438,97 @@ VideoCaptureDevice* DeviceManager::videoCaptureDevice(int id)
  */
 void DeviceManager::updateDeviceList()
 {
-    QList<QByteArray> list;
-
-    GstElement *captureDevice = gst_element_factory_make("v4l2src", NULL);
-    if (captureDevice) {
-        QList<QByteArray> list;
-        list = GstHelper::extractProperties(captureDevice, "device");
-        for (int i = 0; i < list.size() ; ++i) {
-            QByteArray gstId = list.at(i);
-            if (deviceId(gstId) == -1) {
-                m_videoCaptureDeviceList.append(VideoCaptureDevice(this, gstId));
-                emit deviceAdded(deviceId(gstId));
-                m_backend->logMessage(QString("Found new video capture device %0").arg(QString::fromUtf8(gstId)), Backend::Debug, this);
-            }
-        }
-        gst_element_set_state (captureDevice, GST_STATE_NULL);
-        gst_object_unref (captureDevice);
-
-        if (list.size() < m_videoCaptureDeviceList.size()) {
-            //a device was removed
-            for (int i = m_videoCaptureDeviceList.size() -1 ; i >= 0 ; --i) {
-                QByteArray currId = m_videoCaptureDeviceList[i].gstId;
-                bool found = false;
-                for (int k = list.size() -1  ; k >= 0 ; --k) {
-                    if (currId == list[k]) {
-                        found = true;
-                        break;
-                    }
-                }
-                if (!found) {
-                    m_backend->logMessage(QString("Video capture device lost %0").arg(QString::fromUtf8(currId)), Backend::Debug, this);
-                    emit deviceRemoved(deviceId(currId));
-                    m_videoCaptureDeviceList.removeAt(i);
-                }
-            }
-        }
-        list.clear();
-    }
-    //fetch list of current devices
-    GstElement *audioSink= createAudioSink();
+    QList<DeviceInfo> newDeviceList;
+    QList<QByteArray> names;
 
+    /*
+     * Audio output
+     */
+    GstElement *audioSink = createAudioSink();
     if (audioSink) {
         if (!PulseSupport::getInstance()->isActive()) {
             // If we're using pulse, the PulseSupport class takes care of things for us.
-            list = GstHelper::extractProperties(audioSink, "device");
-            list.prepend("default");
+            names = GstHelper::extractProperties(audioSink, "device");
+            names.prepend("default");
         }
 
-        for (int i = 0 ; i < list.size() ; ++i) {
-            QByteArray gstId = list.at(i);
-            if (deviceId(gstId) == -1) {
-                // This is a new device, add it
-                m_audioDeviceList.append(AudioDevice(this, gstId));
-                emit deviceAdded(deviceId(gstId));
-                m_backend->logMessage(QString("Found new audio device %0").arg(QString::fromUtf8(gstId)), Backend::Debug, this);
-            }
+        /* Determine what factory was used to create the sink, to know what to put in the
+         * device access list */
+        GstElementFactory *factory = gst_element_get_factory(audioSink);
+        const gchar *factoryName = gst_plugin_feature_get_name(GST_PLUGIN_FEATURE(factory));
+        QByteArray driver; // means sound system
+        if (!g_strcmp0(factoryName, "alsasink"))       driver = "alsa";
+        if (!g_strcmp0(factoryName, "pulsesink"))      driver = "pulse";
+        if (!g_strcmp0(factoryName, "osssink"))        driver = "oss";
+        if (!g_strcmp0(factoryName, "fakesink"))       driver = "fake";
+        if (driver.isEmpty() && !names.isEmpty())
+            warning() << "Unknown sound system for device" << names.first();
+
+        for (int i = 0; i < names.size(); ++i) {
+            DeviceInfo deviceInfo(this, names[i], DeviceInfo::AudioOutput);
+            deviceInfo.addAccess(DeviceAccess(driver, names[i]));
+            newDeviceList.append(deviceInfo);
         }
 
-        if (list.size() < m_audioDeviceList.size()) {
-            //a device was removed
-            for (int i = m_audioDeviceList.size() -1 ; i >= 0 ; --i) {
-                QByteArray currId = m_audioDeviceList[i].gstId;
-                bool found = false;
-                for (int k = list.size() -1  ; k >= 0 ; --k) {
-                    if (currId == list[k]) {
-                        found = true;
-                        break;
-                    }
-                }
-                if (!found) {
-                    m_backend->logMessage(QString("Audio device lost %0").arg(QString::fromUtf8(currId)), Backend::Debug, this);
-                    emit deviceRemoved(deviceId(currId));
-                    m_audioDeviceList.removeAt(i);
-                }
-            }
+        gst_element_set_state(audioSink, GST_STATE_NULL);
+        gst_object_unref(audioSink);
+    }
+
+    /*
+     * Video capture
+     */
+    GstElement *captureDevice = gst_element_factory_make("v4l2src", NULL);
+    if (captureDevice) {
+        names = GstHelper::extractProperties(captureDevice, "device");
+
+        for (int i = 0; i < names.size(); ++i) {
+            DeviceInfo deviceInfo(this, names[i], DeviceInfo::VideoCapture);
+            deviceInfo.addAccess(DeviceAccess("v4l2", names[i]));
+            newDeviceList.append(deviceInfo);
         }
 
-        gst_element_set_state (audioSink, GST_STATE_NULL);
-        gst_object_unref (audioSink);
+        gst_element_set_state(captureDevice, GST_STATE_NULL);
+        gst_object_unref(captureDevice);
+    }
+
+    /*
+     * Compares the list with the devices available at the moment with the last list. If
+     * a new device is seen, a signal is emitted. If a device dissapeared, another signal
+     * is emitted.
+     */
+
+    // Search for added devices
+    for (int i = 0; i < newDeviceList.count(); ++i) {
+        int id = newDeviceList[i].id();
+        if (!listContainsDevice(m_devices, id)) {
+            // This is a new device, add it
+            m_devices.append(newDeviceList[i]);
+            emit deviceAdded(id);
+
+            debug() << "Found new device" << newDeviceList[i].name();
+        }
+    }
+
+    // Search for removed devices
+    for (int i = m_devices.count() - 1; i >= 0; --i) {
+        int id = m_devices[i].id();
+        if (!listContainsDevice(newDeviceList, id)) {
+            debug() << "Lost device" << m_devices[i].name();
+
+            emit deviceRemoved(id);
+            m_devices.removeAt(i);
+        }
     }
 }
 
-/**
-  * Returns a list of hardware id usable by gstreamer [i.e hw:1,0]
-  */
-const QList<AudioDevice> DeviceManager::audioOutputDevices() const
+bool DeviceManager::listContainsDevice(const QList<DeviceInfo> &list, int id)
 {
-    return m_audioDeviceList;
-}
-
-const QList<VideoCaptureDevice> DeviceManager::videoCaptureDevices() const
-{
-    return m_videoCaptureDeviceList;
+    foreach (const DeviceInfo &d, list) {
+        if (d.id() == id)
+            return true;
+    }
+    return false;
 }
 
 }
 }
-
-QT_END_NAMESPACE
