@@ -15,15 +15,19 @@
     along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#ifndef QT_NO_OPENGL
+#include "glrenderer.h"
+#endif
+
 #include <QtGui/QPainter>
 #include <QtGui/QResizeEvent>
 
 #ifndef QT_NO_OPENGL
 
 #include "backend.h"
+#include "debug.h"
 #include "mediaobject.h"
 #include "qwidgetvideosink.h"
-#include "glrenderer.h"
 #include "qrgb.h"
 #include "videowidget.h"
 
@@ -41,8 +45,6 @@
 # define GL_TEXTURE2    0x84C2
 #endif
 
-QT_BEGIN_NAMESPACE
-
 static void frameRendered()
 {
     static QString displayFps = qgetenv("PHONON_GST_FPS");
@@ -72,7 +74,7 @@ GLRenderer::GLRenderer(VideoWidget* videoWidget) :
     AbstractRenderer(videoWidget)
     , m_glWindow(0)
 {
-    videoWidget->backend()->logMessage("Creating OpenGL renderer");
+    debug() << "Creating OpenGL renderer";
     QGLFormat format = QGLFormat::defaultFormat();
     format.setSwapInterval(1);    // Enable vertical sync on draw to avoid tearing
     m_glWindow = new GLRenderWidgetImplementation(videoWidget, format);
@@ -321,7 +323,5 @@ void GLRenderWidgetImplementation::paintEvent(QPaintEvent *)
 }
 } //namespace Phonon::Gstreamer
 
-QT_END_NAMESPACE
-
 #endif // QT_OPENGL_ES
 #endif // QT_NO_OPENGL
