@@ -162,14 +162,6 @@ void X11Renderer::setOverlay()
 {
     if (m_videoSink && GST_IS_X_OVERLAY(m_videoSink)) {
         WId windowId = m_renderWidget->winId();
-        // Even if we have created a winId at this point, other X applications
-        // need to be aware of it.
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-#warning syncx
-//         QApplication::syncX();
-#else
-        QApplication::syncX();
-#endif
 #if GST_VERSION >= GST_VERSION_CHECK(0,10,31,0)
         gst_x_overlay_set_window_handle(GST_X_OVERLAY(m_videoSink), windowId);
 #else
@@ -182,12 +174,11 @@ void X11Renderer::setOverlay()
 
 void X11Renderer::windowExposed()
 {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-#warning syncx
-//     QApplication::syncX();
-#else
-    QApplication::syncX();
-#endif
+    // This can be invoked within a callchain in an arbitrary thread, so make
+    // sure we call syncX() from the main thread
+    QMetaObject::invokeMethod(m_videoWidget, "syncX",
+                              Qt::QueuedConnection);
+
     if (m_videoSink && GST_IS_X_OVERLAY(m_videoSink))
         gst_x_overlay_expose(GST_X_OVERLAY(m_videoSink));
 }
